@@ -13,14 +13,15 @@ from collections.abc import Iterable
 import sys
 import json
 import multiprocessing
-from urllib import request
+from urllib import request, error
 import tarfile
 import os
+import sys
 
 #required
 import numpy as np
 import h5py as h5
-from scipy import sparse
+from scipy import sparse, stats
 
 PACKAGE_PATH = os.path.dirname(__file__)
 CONFIG_PATH = os.path.join(PACKAGE_PATH, 'config.ini')
@@ -183,10 +184,15 @@ class LISA:
             #http://cistrome.org/~alynch/data/genes.tar.gz
             download_dataset = _config.get('downloads','{species}_{version}'.format(species = self.species, version = REQURED_DATASET_VERSION))
 
-            filename, _ = request.urlretrieve(
-                download_dataset, 
-                os.path.join(self.data_path, self.species + '_data.tar.gz')
-            )
+            try:
+                filename, _ = request.urlretrieve(
+                    download_dataset, 
+                    os.path.join(self.data_path, self.species + '_data.tar.gz')
+                )
+            except error.URLError as err:
+                self.log.append('ERROR: Cannot connect to cistrome.org for data!')
+                self.log.append(err)
+                sys.exit()
             
             self.log.append('Extracting data ...')
 
@@ -257,6 +263,10 @@ class LISA:
         test_statistic = np.sum(weights * np.tan((0.5-p_vals) * np.pi), axis = 1)
         combined_p_value = 0.5 - np.arctan(test_statistic)/np.pi
 
+        for i, (combined, uncombined) in enumerate(zip(combined_p_value, p_vals)):
+            if combined == 0:
+                combined_p_value[i] = stats.combine_pvalues(uncombined)[1] #if cauchy combination p-value is asymtotically small, replace with fisher combination test p-value
+
         return combined_p_value
 
     @staticmethod
@@ -297,14 +307,6 @@ class LISA:
         return background_genes
 
 
-    def _match_background_list(self, query_genes, background_list):
-
-        background_matches = self.all_genes.match_user_provided_genes(background_list)
-        background_genes = background_matches.get_distinct_genes_by_symbol(excluding = query_genes.get_symbols())
-        assert( len(background_genes) > len(query_genes) )
-
-        return background_genes
-             
     def _get_query_and_background_genes(self, query_list, *, background_list = [], num_background_genes = 3000, background_strategy =
         'regulatory', seed = None):
 
@@ -317,7 +319,10 @@ class LISA:
             .format(str(self.max_query_genes), str(len(query_genes)))
 
         if background_strategy == 'provided':
-            background_genes = self._match_background_list(background_list)
+            
+            background_matches = self.all_genes.match_user_provided_genes(background_list)
+            background_genes = background_matches.get_distinct_genes_by_symbol(excluding = query_genes.get_symbols())
+            assert( len(background_genes) > len(query_genes) )
         
         else:
             background_genes = self._sample_background_genes(query_genes, background_strategy = background_strategy, 
@@ -352,7 +357,7 @@ class LISA:
             **assay_pvals
         )
 
-        results = results.sortby('combined_p_value_adjusted', add_rank = True)
+        results = results.sortby('combined_p_value', add_rank = True)
         
         self.log.append('Done!')
         
